#!/usr/bin/env python3
#
# Agent class for single-copy and carry-only routing.
# Copyright (c) 2011-2019, Hiroyuki Ohsaki.
# All rights reserved.
#
# Id: CarryOnly.pm,v 1.11 2015/12/11 08:14:04 ohsaki Exp $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from collections import defaultdict
import math

from perlcompat import die

MAX_RANGE = 50

class CarryOnly():
    def __init__(self,
                 id_=None,
                 scheduler=None,
                 mobility=None,
                 monitor=None,
                 range_=50):
        if id_ is None:
            id_ = len(scheduler.agents) + 1
        if not scheduler:
            die("Scheduler class must be specified.")
        if not mobility:
            die("Mobility class must be specified.")
        if not monitor:
            die("Monitor class must be specified.")
        if range_ > MAX_RANGE:
            die(f"range cannot exceed MAX_RANGE ({MAX_RANGE})")
        self.id_ = id_
        self.scheduler = scheduler
        self.mobility = mobility
        self.monitor = monitor
        self.range_ = range_

        self.last_neighbors = []
        self.received = defaultdict(int)
        self.receive_queue = defaultdict(int)
        self.delivered = defaultdict(int)
        self.tx_count = 0
        self.rx_count = 0
        self.dup_count = 0

        self.scheduler.add_agent(self)

    def __repr__(self):
        name = self.__class__.__name__
        return '{}(id_={!r}, mobility={!r}, range_={!r}'.format(
            name, self.id_, self.mobility, self.range_)

    def msg_src(self, msg):
        src = msg.split('-')[0]
        return int(src)

    def msg_dst(self, msg):
        dst = msg.split('-')[1]
        return int(dst)

    def msg_id(self, msg):
        id_ = msg.split('-')[2]
        return int(id_)

    def zone(self, x=None, y=None):
        """Return the zone (I, J) corresponding the geometry (X, Y)."""
        if x is None:
            x = self.mobility.current[0]
        if y is None:
            y = self.mobility.current[1]
        i = max(0, int(x / MAX_RANGE))
        j = max(0, int(y / MAX_RANGE))
        return i, j

    def cache_zone(self):
        """Record the zone of the current agent in the global zone cache."""
        i, j = self.zone()
        cache = self.scheduler.zone_cache
        cache.setdefault(j, {})
        cache[j].setdefault(i, [])
        self.scheduler.zone_cache[j][i].append(self)

    def neighbors(self):
        """Return neighbor agents within the communication range."""
        cache = self.scheduler.zone_cache
        if not cache:
            die("update_zone() must have been called for zone caching.")

        p = self.mobility.current
        i, j = self.zone()
        neighbors = []
        # check nine zones including/surrounding the current one
        for dj in [-1, 0, 1]:
            if j + dj < 0:
                continue
            for di in [-1, 0, 1]:
                if i + di < 0:
                    continue
                if not cache.get(j + dj, None):
                    continue
                if not cache[j + dj].get(i + di, None):
                    continue
                for agent in self.scheduler.zone_cache[j + dj][i + di]:
                    if agent == self:
                        continue
                    q = agent.mobility.current
                    if abs(p[0] - q[0]) > self.range_:
                        continue
                    if abs(p[1] - q[1]) > self.range_:
                        continue
                    if math.sqrt((p[0] - q[0])**2 +
                                 (p[1] - q[1])**2) > self.range_:
                        continue
                    neighbors.append(agent)
        return neighbors

    def encounters(self):
        """Return encoutered agents (i.e., newly visible agents)."""
        neighbors = self.neighbors()
        encounters = {agent.id_: agent for agent in neighbors}
        for agent in self.last_neighbors:
            try:
                del encounters[agent.id_]
            except KeyError:
                pass
        self.last_neighbors = neighbors
        return list(encounters.values())

    def sendmsg(self, agent, msg):
        """Send a message MSG to the agent AGENT."""
        agent.recvmsg(self, msg)
        self.tx_count += 1
        self.monitor.display_forward(self, agent, msg)
        self.monitor.change_agent_status(self)

    def recvmsg(self, agent, msg):
        """Receive a message MSG from the agent AGENT.  Note that the received
        message is temporally stored in the reception queue."""
        self.receive_queue[msg] += 1
        self.rx_count += 1
        if msg in self.received:
            self.dup_count += 1
        self.monitor.change_agent_status(self)

    def messages(self):
        """Return all received messages."""
        return [msg for msg in self.received if self.received[msg] > 0]

    def pending_messages(self):
        """Return all messages need to be delivered."""
        return [
            msg for msg in self.messages()
            if self.msg_dst(msg) != self.id_ and not msg in self.delivered
        ]

    def accepted_messages(self):
        """Return successfully-received messages by the agent."""
        return [
            msg for msg in self.messages() if self.msg_dst(msg) == self.id_
        ]

    def forward(self):
        """Try to forward carrying messages to all encountered agents."""
        encounters = self.encounters()
        for agent in encounters:
            for msg in self.pending_messages():
                # forward carrying messages only to the destination
                dst = self.msg_dst(msg)
                if agent.id_ != dst:
                    continue
                self.sendmsg(agent, msg)
                del self.received[msg]
                self.delivered[msg] += 1

    def advance(self):
        """Advance the simulation for a delta time."""
        self.mobility.move(self.scheduler.delta)
        self.monitor.move_agent(self)
        self.forward()

    def flush(self):
        """Merge all newly-received messages with existing ones."""
        # NOTE: expand list to avoid run-time error
        for msg in list(self.receive_queue.keys()):
            self.received[msg] += self.receive_queue[msg]
            del self.receive_queue[msg]



#!/usr/bin/env3 python3
#
# A class for dummy underlying path.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#
# $Id: none.py,v 1.2 2018/10/15 13:01:19 ohsaki Exp $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

class NONE:
    def __init__(self, width=1000, height=1000, *args, **kwargs):
        self.width = width
        self.height = height
        self.graph = None



#!/usr/bin/env python3
#
# A dummy class without simulation monitoring.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#
# Id: Null.pm,v 1.14 2015/12/28 10:28:46 ohsaki Exp $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from perlcompat import die

class Null:
    def __init__(self, scheduler=None):
        if scheduler is None:
            die("Scheduler class must be specified.")
        self.scheduler = scheduler
        self.tx_total = 0
        self.rx_total = 0
        self.dup_total = 0
        self.uniq_total = 0
        self.delivered_total = 0
        self.uniq_delivered_total = 0

    def open(self):
        pass

    def update(self):
        pass

    def close(self):
        pass

    def display_path(self, path):
        pass

    def display_agents(self):
        pass

    def move_agent(self, agent):
        pass

    def change_agent_status(self, agent):
        pass

    def display_status(self):
        pass

    def is_delivered(self, agent, msg):
        """Check if the destination of the message MSG is AGENT."""
        return agent.msg_dst(msg) == agent.id_

    def is_duplicate(self, agent, msg):
        """Check if the agent AGENT already has the message MSG."""
        return agent.received.get(msg, None)

    def display_forward(self, src_agent, dst_agent, msg):
        """Update the statistics when the message MSG is forwarded from
        SRC_AGENT to DST_AGENT."""
        self.tx_total += 1
        self.rx_total += 1
        if self.is_duplicate(dst_agent, msg):
            self.dup_total += 1
        if self.is_delivered(dst_agent, msg):
            self.delivered_total += 1
        if self.is_delivered(dst_agent,
                             msg) and not dst_agent.received.get(msg, None):
            self.uniq_delivered_total += 1



#!/usr/bin/env python3
#
# A mobility class for random walk.
# Copyright (c) 2011-2013, Hiroyuki Ohsaki.
# All rights reserved.
#
# Id: RandomWalk.pm,v 1.11 2015/12/09 14:45:23 ohsaki Exp $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import math
import random

from vector_2d import Vector as V
from dtnsim.mobility.fixed import Fixed

class RandomWalk(Fixed):
    def __init__(self, vel_func=None, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        if vel_func is None:
            vel_func = lambda: 1.0  # 1.0 [m/s] by default
        self.vel_func = vel_func
        self.velocity = None
        self.wait = None

    def update_velocity(self):
        """Update agent's velocity using the velocity function."""
        vel = self.vel_func()
        theta = random.uniform(0, 2 * math.pi)
        self.velocity = vel * V(math.cos(theta), math.sin(theta))
        self.wait = False

    def move(self, delta):
        """Move the agent for the duration of DELTA."""
        self.update_velocity()
        self.current += self.velocity * delta



#!/usr/bin/env python3
#
# A dummy class without simulation monitoring.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#
# Id: Log.pm,v 1.12 2015/12/30 02:55:06 ohsaki Exp $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from dtnsim.monitor.null import Null

class Log(Null):
    # NOTE: this code must be invoked before calling flush()
    def display_forward(self, src_agent, dst_agent, msg):
        """Update the statistics when the message MSG is forwarded from
        SRC_AGENT to DST_AGENT."""
        super().display_forward(src_agent, dst_agent, msg)
        stat = []
        if self.is_delivered(dst_agent, msg):
            stat.append('delivered')
        if self.is_duplicate(dst_agent, msg):
            stat.append('dup')
        stat_str = ','.join(stat)
        print('\t'.join([
            str(self.scheduler.time), 'forward',
            str(src_agent.id_),
            str(dst_agent.id_), msg, stat_str
        ]))
        print('\t'.join([
            str(self.scheduler.time), 'status',
            str(self.tx_total),
            str(self.rx_total),
            str(self.dup_total),
            str(self.uniq_total),
            str(self.delivered_total),
            str(self.uniq_delivered_total)
        ]))



#!/usr/bin/env python3
#
# Agent class implementing infinite-copy ProPHET routing.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#
# Id: ProPHET.pm,v 1.8 2016/01/19 17:25:53 ohsaki Exp $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from collections import defaultdict

from dtnsim.agent.epidemic import Epidemic

P_INIT = 0.75
BETA = 0.25
GAMMA = 0.999

class ProPHET(Epidemic):
    def __init__(self, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        self.d_pred = defaultdict(float)

    def update_d_pred(self, encounters):
        """Update delivery predictabilities to all destination agents."""
        encoutered_with = defaultdict(int)
        for agent in encounters:
            # refresh delivery predictability
            neighbor = agent.id_
            encoutered_with[neighbor] += 1
            self.d_pred[neighbor] += (1 - self.d_pred[neighbor]) * P_INIT
            # update delivery predictabilities using transitivity
            for dst in agent.d_pred:
                if dst == self.id_:
                    continue
                self.d_pred[dst] += (
                    1 - self.d_pred[dst]
                ) * self.d_pred[neighbor] * agent.d_pred[dst] * BETA

        # decay delivery predictabilities to all destinations
        # FIXME: do not decay for encountered agents
        for dst in self.d_pred:
            if encoutered_with[dst]:
                continue
            self.d_pred[dst] *= GAMMA

    def should_forward(self, agent, msg):
        """Check whether the message MSG should be forwarded to an encountered
        agent AGENT."""
        # do not forward if the neighbor already has the message
        if agent.received[msg]:
            return
        # forward only when neighbor is closer to the destination than itself
        dst = self.msg_dst(msg)
        if dst not in agent.d_pred:
            return
        return agent.d_pred[dst] > self.d_pred[dst]

    def forward(self):
        """Try to deliver its carrying messages to all encountered agents."""
        encounters = self.encounters()
        self.update_d_pred(encounters)
        for agent in encounters:
            for msg in self.pending_messages():
                dst = self.msg_dst(msg)
                if agent.id_ == dst:
                    self.sendmsg(agent, msg)
                    del self.received[msg]
                    self.delivered[msg] += 1
                else:
                    if not self.should_forward(agent, msg):
                        continue
                    self.sendmsg(agent, msg)



#!/usr/bin/env python3
#
# An agent class implementing P-BCAST (PUSH-based BroadCAST).
# Copyright (c) 2011-2015, Hiroyuki Ohsaki.
# All rights reserved.
#
# Id: P_BCAST.pm,v 1.32 2015/12/09 14:45:23 ohsaki Exp $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from dtnsim.agent.carryonly import CarryOnly

class P_BCAST(CarryOnly):
    """Try to deliver its carrying messages to all encountered agents."""

    def forward(self):
        encounters = self.encounters()
        for agent in encounters:
            for msg in self.received:
                self.sendmsg(agent, msg)



#!/usr/bin/env python3
#
# A mobility class for simulating FullMixed model.
# Copyright (c) 2016-2018, Hiroyuki Ohsaki.
# All rights reserved.
#
# Id: FullMixed.pm,v 1.5 2016/07/03 10:32:54 ohsaki Exp $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# NOTE: communication range must be set to a sufficiently small value

import random

from dtnsim.mobility.fixed import Fixed
from vector_2d import Vector as V

NODE_SEPERATION = 25
max_id = 0

class FullMixed(Fixed):
    def coordinate_for(self, n):
        """Return the coordinate of N-th agent.  Agents are placed such that
        any pair of agents are not reachable."""
        return V(n * NODE_SEPERATION, self.height / 2)

    def __init__(self, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        global max_id
        max_id += 1
        self.id_ = max_id
        self.rate = 1 / 1000  # encounter rate
        self.visit_list = []
        self.current = self.coordinate_for(self.id_)

    def move(self, delta):
        """Move the agent for the duration of DELTA."""
        # randomly choose other nodes to encouter
        for to_id in range(1, max_id + 1):
            if to_id >= self.id_:
                break
            if random.uniform(0, 1) <= self.rate / delta:
                self.visit_list.append(to_id)

        # visit other to-count agents sequentially
        if self.visit_list():
            to_id = self.visit_list.pop(0)
            self.current = self.coordinate_for(to_id)
        else:
            self.current = self.coordinate_for(self.id_)



#!/usr/bin/env python3
#
# An agent class implementing SA-BCAST (Self-Adaptive BroadCAST).
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#
# Id: SA_BCAST.pm,v 1.16 2015/12/09 14:45:23 ohsaki Exp $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import random

from dtnsim.agent.p_bcast import P_BCAST

# default control parameters for SA-BCAST
C = 1.5
MIN_P = 0.01
N_TH = 50

class SA_BCAST(P_BCAST):
    def __init__(self, c=C, min_p=MIN_P, n_th=N_TH, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        self.c = c
        self.min_p = min_p
        self.n_th = n_th

    def forward(self):
        """Try to deliver its carrying messages to all encountered agents."""
        encounters = self.encounters()
        if not encounters:
            return

        # forward only when N-th% of neighbors has changed
        neighbors = self.neighbors()
        change_ratio = len(encounters) / len(neighbors)
        if change_ratio < self.n_th / 100:
            return

        for agent in encounters:
            for msg in self.received:
                # change forwarding probability based on the number of duplicates
                ndups = max(self.received[msg] - 1, 0)
                p = max(1 / self.c**ndups, self.min_p)
                if random.uniform(0, 1) > p:
                    continue
                self.sendmsg(agent, msg)



#!/usr/bin/env python3
#
# A mobility class for CRWP (Constrained Random Waypoint) model on a graph.
# Copyright (c) 2011, Hiroyuki Ohsaki.
# All rights reserved.
#
# $Id: crwp.py,v 1.2 2018/10/15 12:48:51 ohsaki Exp ohsaki $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import random

from dtnsim.mobility.graph.randomwalk import RandomWalk

class CRWP(RandomWalk):
    def __init__(self, pause_func=None, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        if pause_func is None:
            pause_func = lambda: 0.0  # no pause time by default
        self.pause_func = pause_func
        self.goal = None
        self.goal_edge = None
        self.goal_offset = None
        self.wait = False
        self.pick_goal()
        self.update_velocity()

    def update_velocity(self):
        """Update agent's velocity."""
        self.velocity = self.vel_func()

    def update_goal_cache(self):
        """Compute and store the goal coordinate for later use."""
        self.goal = self.get_coordinate(self.goal_edge, self.goal_offset)

    def select_route(self):
        """Select the link through which the agent should proceed to reach its
        destination (goal).  This code assumes that the agent is just on the
        vertex and the graph is plannar."""
        # NOTE: this code assumes that the current position of the agent is is V
        u, v = self.current_edge
        pv = self.vertex_coordinate(v)
        next_ = u  # for fail safe
        min_angle = None
        # find the vertex W whose angle is closest to the goal
        for w in self.path.graph.neighbors(v):
            # never goes back to the coming direction
            if w == u:
                continue
            pw = self.vertex_coordinate(w)
            angle = self.angle_between_vectors(self.goal - pv, pw - pv)
            if min_angle is None or angle < min_angle:
                min_angle = angle
                next_ = w
        self.move_to_point([v, next_], 0)

    def reverse_current_if_necessary(self):
        """Reverse the current direction if the opposite is closer to the goal."""
        u, v = self.current_edge
        goal = self.goal - self.current
        via_v = self.vertex_coordinate(v) - self.current
        via_u = self.vertex_coordinate(u) - self.current
        if self.angle_between_vectors(
                goal, via_u) < self.angle_between_vectors(goal, via_v):
            length = self.edge_length(self.current_edge)
            self.current_edge = [v, u]
            self.current_offset = length - self.current_offset

    def pick_goal(self):
        """Randomly choose the location to go."""
        edge, offset = self.random_point()
        self.goal_edge = edge
        self.goal_offset = offset
        self.update_goal_cache()
        self.reverse_current_if_necessary()

    def move(self, delta):
        """Move the agent for the duration of DELTA."""
        # sleep until wait timer expires
        self.wait = max(self.wait - delta, 0)
        if self.wait > 0:
            return

        # advance the agent by SPEED * DELTA
        step = self.velocity * delta
        self.current_offset += step
        self.update_current_cache()

        l = self.edge_length(self.current_edge)
        # if close enough to the goal, randomly choose the next goal
        if abs(self.goal - self.current) < step:
            self.wait = self.pause_func()
            self.pick_goal()
            self.update_velocity()
        elif abs(self.current_offset - l) < step or self.current_offset > l:
            self.select_route()



#!/usr/bin/env3 python3
#
# A class for Grid-style underlying path.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#
# $Id: grid.py,v 1.2 2018/10/15 12:59:08 ohsaki Exp ohsaki $
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import graph_tools

from dtnsim.path.line import Line
from vector_2d import Vector as V

class Grid(Line):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # create a grid topology
        g = graph_tools.Graph(directed=False)
        self.graph = g
        g.create_graph('lattice', 2, self.size)

        # save the positions of vertices as Vector object
        for j in range(1, self.size + 1):
            for i in range(1, self.size + 1):
                v = g._lattice_vertex(2, self.size, i, j)
                x, y = ((0.5 + i - 1) / self.size * self.width,
                        (0.5 + j - 1) / self.size * self.height)
                g.set_vertex_attribute(v, 'xy', V(x, y))
        self.compute_edge_lengths()
